#region Using directives

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

#endregion

namespace hammer
{
    public delegate void MruOpenEventHandler(object sender, MruOpenEventArgs e);

    public class MruOpenEventArgs : EventArgs
    {
        private string filename;

        public MruOpenEventArgs(string filename)
        {
            this.filename = filename;
        }

        public string FileName { get { return filename; } }
    }

    public class MruList
    {
        private string keyname, valuename;
        private List<string> fileList;
        private ToolStripMenuItem mruParent;

        public MruList(int capacity, string keyname, string valuename,
            ToolStripMenuItem mruParent)
        {
            this.fileList = new List<string>(capacity);
            this.keyname = keyname;
            this.valuename = valuename;
            this.mruParent = mruParent;
            string[] values = (string[])Microsoft.Win32.Registry.GetValue(
                keyname, valuename, new string[0]);
            if (values != null)
                for (int i = 0; i < Math.Min(capacity, values.Length); i++)
                    this.fileList.Add(values[i]);
            CreateMenu();
        }

        public event MruOpenEventHandler FileOpen;

        public int Capacity { get { return fileList.Capacity; } }

        public void OpenFile(string filename)
        {
            for (int i = 0; i < fileList.Count; i++)
                if (string.Compare(filename, fileList[i], true) == 0)
                {
                    fileList.RemoveAt(i);
                    ToolStripMenuItem item = (ToolStripMenuItem)mruParent.DropDownItems[i];
                    mruParent.DropDownItems.Remove(item);
                    if (System.IO.File.Exists(filename))
                    {
                        fileList.Insert(0, filename);
                        mruParent.DropDownItems.Insert(0, item);
                    }
                    else
                        item.Dispose();
                    mruParent.Enabled = mruParent.DropDownItems.Count > 0;
                    Microsoft.Win32.Registry.SetValue(keyname, valuename,
                        fileList.ToArray());
                    return;
                }
            while (fileList.Count >= fileList.Capacity)
                fileList.RemoveAt(fileList.Count - 1);
            fileList.Insert(0, filename);
            Microsoft.Win32.Registry.SetValue(keyname, valuename,
                fileList.ToArray());
            CreateMenu();
        }

        private void fileOpen(object sender, EventArgs e)
        {
            ToolStripMenuItem item = (ToolStripMenuItem)sender;
            string filename = item.Text;
            if (System.IO.File.Exists(filename))
            {
                MruOpenEventHandler eh = FileOpen;
                if (eh != null)
                    eh(this, new MruOpenEventArgs(filename));
            }
            else
                OpenFile(filename);
        }

        private void CreateMenu()
        {
            if (mruParent != null)
            {
                mruParent.DropDownItems.Clear();
                for (int i = 0; i < fileList.Count; i++)
                {
                    string filename = fileList[i];
                    ToolStripMenuItem item = new ToolStripMenuItem(filename, null, fileOpen);
                    mruParent.DropDownItems.Add(item);
                }
            }
            mruParent.Enabled = mruParent.DropDownItems.Count > 0;
        }
    }
}
