using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;

namespace hammer
{
    public partial class PropsDialog : Form
    {
        public static void Execute(string fileName, IntSight.Controls.CodeEditor editor)
        {
            using (PropsDialog form = new PropsDialog())
            {
                form.Init(fileName, editor);
                form.ShowDialog();
            }
        }

        private void Init(string fileName, IntSight.Controls.CodeEditor editor)
        {
            txDiskSize.Text = String.Empty;
            txCreated.Text = String.Empty;
            txModified.Text = String.Empty;
            txLastAccess.Text = String.Empty;
            if (String.IsNullOrEmpty(fileName))
            {
                txFileName.Text = "Untitled";
                FileName = "(not saved)";
            }
            else
            {
                txFileName.Text = Path.GetFileName(fileName);
                FileName = fileName;
                FileInfo info = new FileInfo(fileName);
                if (info.Exists)
                {
                    txDiskSize.Text =
                        GetSizeString("byte", "bytes", info.Length) + " (disk)";
                    txCreated.Text = info.CreationTime.ToLongDateString() +
                        ", " + info.CreationTime.ToLongTimeString();
                    txModified.Text = info.LastWriteTime.ToLongDateString() +
                        ", " + info.LastWriteTime.ToLongTimeString();
                    txLastAccess.Text = info.LastAccessTime.ToLongDateString() +
                        ", " + info.LastAccessTime.ToLongTimeString();
                }
                toolTip.SetToolTip(txPath, fileName);
            }
            txPath.Text = String.Empty;
            int documentSize = editor.GetDocumentSize();
            int undoSize = editor.GetUndoStackSize();
            if (documentSize <= 2)
            {
                txLines.Text = "(empty)";
                txMemorySize.Text = "(empty)";
                if (undoSize > 0)
                    toolTip.SetToolTip(txMemorySize, "Estimated undo/redo stack size: " +
                        GetSizeString("byte", "bytes", undoSize));
            }
            else
            {
                txLines.Text = GetSizeString("line", "lines", editor.LineCount);
                txMemorySize.Text =
                    GetSizeString("byte", "bytes", documentSize) + " (editor)";
                toolTip.SetToolTip(txMemorySize, "Estimated undo/redo stack size: " +
                    GetSizeString("byte", "bytes", undoSize));
            }
        }

        private static string GetSizeString(string singular, string plural, long size)
        {
            return size.ToString("#,0") + " " + (size == 1 ? singular : plural);
        }

        private string FileName;

        private PropsDialog()
        {
            InitializeComponent();
            Opacity = (MainForm.GlobalOpacity - 1) / 100.0;
        }

        private void txPath_Paint(object sender, PaintEventArgs e)
        {
            if (!String.IsNullOrEmpty(FileName))
            {
                StringFormat fmt = new StringFormat(StringFormat.GenericTypographic);
                fmt.Trimming = StringTrimming.EllipsisPath;
                fmt.LineAlignment = StringAlignment.Center;
                fmt.Alignment = StringAlignment.Near;
                e.Graphics.DrawString(FileName,
                    txPath.Font, Brushes.Black, txPath.ClientRectangle, fmt);
            }
        }
    }
}